// Note: this file is part of a header-only library. See Readme.md

#ifndef BSETS2_MATRIX_H
#define BSETS2_MATRIX_H

#include "options.h"
#include "FFel.hpp"
#include "Point.hpp"
#include <iostream>

////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////

class Matrix {
private:

    Matrix() = default;

public:
    FFel m[9]{}; // Hierin komen de matrixelementen

    Matrix(const FFel &a, const FFel &b, const FFel &c, const FFel &d, const FFel &e, const FFel &f, const FFel &g,
           const FFel &h, const FFel &i) : m {a,b,c,d,e,f,g,h,i} {};

    FFel det() const {
        return (m[0] * m[4] * m[8] + m[1] * m[5] * m[6] + m[2] * m[3] * m[7]
                - m[2] * m[4] * m[6] - m[1] * m[3] * m[8] - m[0] * m[5] * m[7]);
    }

    // Copmutes the adjoint of this matrix. Because these are projectivities, the adjoint can be used as an inverse.
    Matrix adjoint() const {
        return Matrix(m[4] * m[8] - m[5] * m[7], m[2] * m[7] - m[1] * m[8], m[1] * m[5] - m[2] * m[4],
                      m[5] * m[6] - m[3] * m[8], m[0] * m[8] - m[2] * m[6], m[2] * m[3] - m[0] * m[5],
                      m[3] * m[7] - m[4] * m[6], m[1] * m[6] - m[0] * m[7], m[0] * m[4] - m[1] * m[3]);
    }

    /*
    //This matrix * other matrix
    Matrix matrixProduct(const Matrix &other) const{
        return Matrix(m[0] * other.m[0] + m[1] * other.m[3] + m[2] * other.m[6],
                      m[0] * other.m[1] + m[1] * other.m[4] + m[2] * other.m[7],
                      m[0] * other.m[2] + m[1] * other.m[5] + m[2] * other.m[8],
                      m[3] * other.m[0] + m[4] * other.m[3] + m[5] * other.m[6],
                      m[3] * other.m[1] + m[4] * other.m[4] + m[5] * other.m[7],
                      m[3] * other.m[2] + m[4] * other.m[5] + m[5] * other.m[8],
                      m[6] * other.m[0] + m[7] * other.m[3] + m[8] * other.m[6],
                      m[6] * other.m[1] + m[7] * other.m[4] + m[8] * other.m[7],
                      m[6] * other.m[2] + m[7] * other.m[5] + m[8] * other.m[8]);
    }
     */

    //Bepaal het beeld van punt p onder deze matrix. // rijvector vermenigvuldigen met matrix: p*M
    Point operator()(const Point &p) const {
        return Point{
                (p.x * m[0] + p.y * m[3] + p.z * m[6]),
                (p.x * m[1] + p.y * m[4] + p.z * m[7]),
                (p.x * m[2] + p.y * m[5] + p.z * m[8])
        };
    }

    bool operator==(const Matrix &other) const;

    void printMatrix() const;

    friend class Group; // must access private constructor
};

////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

void Matrix::printMatrix() const {
    std::cout << "-------" << std::endl;
    std::cout << m[0].index() << " " << m[1].index() << " " << m[2].index() << std::endl;
    std::cout << m[3].index() << " " << m[4].index() << " " << m[5].index() << std::endl;
    std::cout << m[6].index() << " " << m[7].index() << " " << m[8].index() << std::endl;
    std::cout << "-------" << std::endl;
}


bool Matrix::operator==(const Matrix &other) const {
    for (int i = 0; i < 9; ++i) {
        if (m[i] != other.m[i]) {
            return false;
        }
    }
    return true;
}


#endif //BSETS2_MATRIX_H
