// Note: this file is part of a header-only library. See Readme.md

#ifndef BSETS2_ORBITPARTITION_H
#define BSETS2_ORBITPARTITION_H

#include <vector>
#include "options.h"
#include "Matrix.hpp"
#include "PointSet.hpp"

////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////


class OrbitPartition {
public:

    int orbitArray[NOP] = {0,}; //orbitnummers
    bool first[NOP] = {false,}; // is first in its orbit? NOTE: not in original

    /*
     * Create the orbit partition for the given group
     */
    explicit OrbitPartition(const std::vector<const Matrix *> &stabgroup);

    explicit OrbitPartition(); //this is a copy paste from algorithm 1 but should never be called in this algorithm.
};

////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

OrbitPartition::OrbitPartition(const std::vector<const Matrix *> &stabgroup) {
    std::fill_n(orbitArray, NOP, -1);
    int counter = 0;

    for (int i = 0; i < NOP; ++i) {
        if (orbitArray[i] < 0) {//Dan heb je zijn orbit nog niet bepaald, en hij is de eerste in zijn orbit.
            first[i] = true;
            for (const Matrix *pMatrix : stabgroup) {
                const Matrix &matrix = *pMatrix;
                orbitArray[matrix(Point::list[i]).index()] = counter;
            }
            counter++;
        }
    }
}

OrbitPartition::OrbitPartition() {
    std::fill_n(orbitArray, NOP, 2);
    orbitArray[0]= 0;
    orbitArray[1] = 0;
    orbitArray[2] = 0; // a blocking set always has a 3-secant
    //orbitArray[3] = 0; //hypothese
    for (int i = 3; i < Q+1; ++i) {
        orbitArray[i]=1;
    }
    first[0] = true;
    first[3] = true;
    //first[4] = true; //hypothese
    first[Q+1] = true;
}

#endif //BSETS2_ORBITPARTITION_H

