// Note: this file is part of a header-only library. See Readme.md

#ifndef BSETS2_POINT_H
#define BSETS2_POINT_H

#include <cassert>
#include "FFel.hpp"

////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////

class Point {
private:
    Point() = default;

    //Verwarrend geimplementeerd. Eigenlijk zou ik een klasse moeten invoeren voor mijn rechten. Maar rechten en punten spelen toch dezelfde rol dus bij deze.
    // Die other point stelt dus een rechte voor (met dezelfde coordinaatgetallen als other). Deze methode checkt of this point incident is met die rechte.
    bool ptIncLine(const Point &other) const {
        return (x * other.x + y * other.y + z * other.z).isZero();
    }

public:
    FFel x;
    FFel y;
    FFel z;

    Point(const FFel &x, const FFel &y, const FFel &z) : x(x), y(y), z(z) {}

    //Geschreven zodanig dat de coordinaat (x,y,z) nog niet in vereenvoudigde vorm hoeft te staan; maar ze moeten wel al mod q staan (dus in {0,...,q-1} liggen).
    int index() const;

private:
    // non-const versions of public static arrays
    static Point list_private[NOP];
    static int pIl_private[NOP][Q + 1];
    
public:
    /** All points in the plane */
    static const Point (&list)[NOP];
    
    /** For every point, indexes of all lines through that point */
    static const int (&pIl)[NOP][Q + 1];

private:
    static bool initialize();

    static bool initialized;

    friend class PointSet; // must access private constructor

};

////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

// initialize static variables of point

Point Point::list_private[NOP] {};
const Point (& Point::list)[NOP] = list_private;

int Point::pIl_private[NOP][Q + 1]{};
const int (& Point::pIl)[NOP][Q+1] = pIl_private;

bool Point::initialize() {

    //Punten in list steken (zelfde volgorde als in Javaproject)
    int pos = 0;

    list_private[pos++] = Point{0, 0, 1}; // implicit conversion of 0 and 1

    for (const auto &element : FFel::elements) {
        list_private[pos++] = Point{0, 1, element};
    }

    for (const auto &el1 : FFel::elements) {
        for (const auto &el2 : FFel::elements) {
            list_private[pos++] = Point(1, el1, el2);
        }
    }

    assert (pos == NOP);

    // initialize pIl
    for (int k = 0; k < NOP; ++k) { // k loopt over alle punten
        int counter = 0;
        for (int i = 0; i < NOP; ++i) { // i loopt over alle lijnen
            if (list_private[k].ptIncLine(list_private[i])) {
                pIl_private[k][counter ++] = i;
            }
        }
        assert (counter == Q+1);
    }

    return true;
}

bool Point::initialized = Point::initialize();

int Point::index() const {

    if (x.isZero()) {
        if (y.isZero()) {
            return 0;
        } else {
            //y en z vermenigvuldigen met de inverse van y geeft 01z'. Het rangnummer van 01z' is 1+z'
            FFel product =  z / y;
            return (z/y).index() + 1;
        }
    } else {
        if (x.isOne()) { // Dan staat de coordinaat van het punt in de gewenste vorm (begint met 1)
            return (Q + 1) + z.index() + y.index() * Q;
        } else { // De coordinaat van het punt start met een getal >1. Ik vermenigvuldig alle coordinaatgetallen met Xinvers zodat je wel start met 1.
            return (z/x).index() + (y/x).index() * Q + Q + 1;
        }
    }
}

#endif //BSETS2_POINT_H
