// Code specific to the finite field of 8 elements

/* We use a primitive element α such that α^3 = α + 1
 * The field elements are numbered
 *     0, 1, α, α^2, α^3, α^4, α^5, α^6
 * to make the multiplication table cyclic.
 * The same elements, listed in the same order, can also be written as
 *     0, 1, α, α^2, α+1, α^2+α, α^2+α+1, α^2+1
 * which is used to set up the addition table
 */

static int add_table[Q][Q] = {
        {0, 1, 2, 3, 4, 5, 6, 7},
        {1, 0, 4, 7, 2, 6, 5, 3},
        {2, 4, 0, 5, 1, 3, 7, 6},
        {3, 7, 5, 0, 6, 2, 4, 1},
        {4, 2, 1, 6, 0, 7, 3, 5},
        {5, 6, 3, 2, 7, 0, 1, 4},
        {6, 5, 7, 4, 3, 1, 0, 2},
        {7, 3, 6, 1, 5, 4, 2, 0}
};

static int mul_table[Q][Q] = {
        {0, 0, 0, 0, 0, 0, 0, 0},
        {0, 1, 2, 3, 4, 5, 6, 7},
        {0, 2, 3, 4, 5, 6, 7, 1},
        {0, 3, 4, 5, 6, 7, 1, 2},
        {0, 4, 5, 6, 7, 1, 2, 3},
        {0, 5, 6, 7, 1, 2, 3, 4},
        {0, 6, 7, 1, 2, 3, 4, 5},
        {0, 7, 1, 2, 3, 4, 5, 6}
};

const FFel FFel::elements[Q] = {FFel(0, true), FFel(1, true), FFel(2, true), FFel(3, true),
                        FFel(4, true), FFel(5, true), FFel(6, true), FFel(7, true)};

constexpr int CHARACTERISTIC = 2;

constexpr int LOWERBOUNDB = 13;
constexpr int UPPERBOUNDMINB = 23;
