Generator for minimal blocking sets in small Desarguesian projective planes
===

This is the source code of the generation program used in the paper

* "Classification of minimal blocking sets in small Desarguesian projective planes",
  *Journal of Combinatorial Designs*, Vol 30(8), 2022.

The program is written in C++ and was developed using the CLion IDE which uses
`cmake` as build tool and `clang` as compiler. (It should be possible to rebuild
the software outside of CLion, as long as you have `cmake` and `clang` available.)

A few unit tests are provided with the source code - see below - but they are not
needed to be able to run the main program.

#### Main program

The main program - called `main` - generates all minimal blocking sets in the
Desarguesian plane of a given order. Currently orders 3, 4, 5, 6, 7, and 8 are supported.

You must specify the order you wish to run the program for **before** building
the program. (A rebuild is needed everytime you change the order.) This order
is specified on line 4 of the file `options.h`

When run without parameters, the program generates all *minimal* blocking sets
up to (projective) *equivalence*. 
The case of order 8 takes approximately 2 weeks of computer time. 
You may however provide the program with parameters to indicate that you only want a part of the results.
This way the program can be run in parallel.

Typically you would run the following versions of the program in parallel:

    main 7 0 10
    main 7 1 10
    main 7 2 10
    ...
    main 7 9 10

The last parameter (10) indicates in how many parts the generation should be split,
the second parameter (0..9) indicates which of these parts you want to generate and the
first parameter (7) indicates at which depth the search tree should be split.
(Note that any blocking set with size smaller than or equal to this depth will be generated several times.)

#### Output format

The program generates some statistics on the *standard error* channel. The blocking sets themselves
are sent to *standard output*, one line per blocking set. Each line
lists the points as numbers terminated by dashes (e.g. `0-1-2-5-6-9-10-`). The points are
numbered according to their coordinates as follows:

    (0,0,1) → 0
    (0,1,z) → 1 + z
    (1,y,z) → q + 1 + q·y + z

Where `q` is the order of the field. (See for instance `Point::index()` in `Point.hpp`.)

The field elements `y`, `z` in the above are numbered 0,1,2,… according to the following order:
* For q=3: 0,1,2 mod 3
* For q=4: 0,1,α,α² where α²=α+1
* For q=5: 0,1,2,3,4 mod 5
* For q=7: 0,1,2,3,4,5,6 mod 7
* For q=8: 0,1,α,α²,α³,… where α³=α+1

(See files `ff3.h` … `ff8.h`.)

#### Unit tests

To build and execute the unit tests you need a directory `Google_tests/lib` which is not included
in the repository. 
Clone Google Test from the [official repository](https://github.com/google/googletest). 
Link that folder (or make a copy) to `Google_tests/lib`.

#### Implementation note

To solve earlier problems with the *static initialization order fiasco* and to provide
better compiler optimization, the main library of is largely implemented as a 
[header only](https://en.wikipedia.org/wiki/Header-only) project. 
Files used as part of this header only scheme are given the extension `.hpp`
(combination of `.h` and `.cpp`).
 
