#include <iostream>
#include <iomanip>
#include <chrono>
#include <Generator.hpp>
#include "options.h"

// clocks - see https://stackoverflow.com/questions/17432502/how-can-i-measure-cpu-time-and-wall-clock-time-on-both-linux-windows

static Generator generator;

int main(int argc, char *argv[]) {

    // registering clocks
    std::clock_t startcputime = std::clock();
    auto wcts = std::chrono::system_clock::now();

    // doing all the work - output to std::cout
    generator.processArgs (argc, argv);
    generator.generate();

    // Report statistics to standard error
    std::cerr << "Minimal blocking sets of PG(2," << Q << ") are: " << std::endl;
    for (int i = 0; i <= NOP; i++) {
        std::cerr << std::setw(3) << i << ": " << generator.nrMinBlockingSetsOfSize[i] << std::endl;
    }

    // showing time used
    double cpu_duration = (double) (std::clock() - startcputime) / CLOCKS_PER_SEC;
    std::cerr << "Finished in " << cpu_duration << " seconds [CPU Clock] " << std::endl;

    std::chrono::duration<double> wctduration = (std::chrono::system_clock::now() - wcts);
    std::cerr << "Finished in " << wctduration.count() << " seconds [Wall Clock]" << std::endl;
    return 0;
}
