#include <iostream>
#include <iomanip>
#include <chrono>
#include <Group.hpp>

// Reads a file of point sets (each line of the form 1-2-3-7-9-11-...)
// and produces the following statistics:
//
//  - size
//  - size of automorphism group

static Group group;

int main(int argc, char *argv[]) {

    // registering clocks
    std::clock_t startcputime = std::clock();
    auto wcts = std::chrono::system_clock::now();

    // input from std::cin - output to std::cout
    std::string line;
    while (std::getline(std::cin, line)) {
        std::istringstream iss(line);
        PointSet set {};
        char ch;
        int pt;
        int count = 0;
        iss >> pt >> ch; // ch is always '-'
        while (!iss.eof()) {
            set.add(pt);
            count ++;
            iss >> pt >> ch; // ch is always '-'
        }

        std::cout << count << " "
                  << group.stabilizer(set).size()
                  << " " << line
                  << std::endl;
    }

    // showing time used
    double cpu_duration = (double) (std::clock() - startcputime) / CLOCKS_PER_SEC;
    std::cerr << "Finished in " << cpu_duration << " seconds [CPU Clock] " << std::endl;

    std::chrono::duration<double> wctduration = (std::chrono::system_clock::now() - wcts);
    std::cerr << "Finished in " << wctduration.count() << " seconds [Wall Clock]" << std::endl;
    return 0;
}

